
Object subclass: #Product
    instanceVariableNames: 'name availableQuantity unitVolume demandLog '
    classVariableNames: ''
    poolDictionaries: ''!

Product comment: 'Diese Klasse entspricht der Klasse Produkt aus der OBA-Analyse.

Instanzvariablen:
	name <Symbol>
		Die Produktbezeichnung innerhalb des Unternehmenssortiments, ''Name''.
	availableQuantity <Number>
		Im Lager aktuell verfuegbare Menge des Produktes, ''Bestand''.
	unitVolume <Number>
		Raumbedarf des Produktes pro Mengeneinheit, ''Volumen''.
	demandLog <OrderedCollection>
		Menge der Bestellposten fuer das Produkt in der Reihenfolge
		des Eingangs, ''Nachfragejournal''.
	'!

!Product class publicMethods !

name: aSymbol
	"Ein neues Produkt wird erstellt."
	| product |
	product := super new initialize.
	product name: aSymbol.
	^product!

new
	^self shouldNotImplement! !

Product class categoriesFor: #name: are:((OrderedCollection new)
	add: 'Create';
	yourself)!
Product class categoriesFor: #new are:((OrderedCollection new)
	add: 'Create';
	yourself)!

!Product publicMethods !

add: aNumber
	"Entspricht der Dienstleistung 'Bestandsaenderung', Teil 1."
	availableQuantity := availableQuantity + aNumber!

availableQuantity
	"Zugriff 'Bestand'."
	"Im Lager aktuell verfuegbare Mengen des Produkts."
	^availableQuantity!

initialize
	"Instanzvariablen werden initialisiert."
	name := nil.
	availableQuantity := 0.
	unitVolume := 1.
	demandLog := SortedCollection sortBlock:
					 [:position1 :position2 | position1 date <= position2 date].
	!

logOrderPosition: anOrderPosition
	"Entspricht der Dienstleistung 'Verbuchung-Nachfrage'."
	"anOrderPosition wird verbucht,
	um die Nachfragerate berechnen zu koennen."
	demandLog add: anOrderPosition!

name
	"Zugriff 'Name'."
	"Die Produktbezeichnung wird zurueckgegeben."
	^name!

name: aSymbol
	"Die Produktbezeichnung wird festgelegt."
	name := aSymbol!

printString
	"Gibt die im Lager verfuegbare Menge eines Produktes aus."
	^'P# ' , (self name asString , ': ' , self availableQuantity printString) , ' avail.'!

release
	"Loescht Datenspeicher eines Objekts."
	name := nil.
	availableQuantity := nil.
	unitVolume := nil.
	demandLog := nil.
!

remove: aNumber
	"Entspricht der Dienstleistung 'Bestandsaenderung', Teil 2."
	availableQuantity := availableQuantity - aNumber!

unitVolume
	"Zugriff 'Einheitdvolumen'."
	"Der Raumbedarf des Produkts wird zurueckgegeben."
	^unitVolume!

unitVolume: aNumber
	"Der Raumbedarf des Produkts wird festgelegt."
	unitVolume := aNumber! !

!Product privateMethods !

demandRate
	"Private - Liefert die Nachfragerate nach dem Produkt.
	Es wird der Durchschnittsbedarf ueber alle
	Bestellungen hinweg verwendet. Der Bedarf
	der ersten Bestellung ist abzuziehen, da diese
	lediglich das erste Datum fuer die Berechnung
	des Zeitraummes liefert."
	| totalDemand totalTime |
	demandLog size <= 1 ifTrue:
		[^self error: 'Unsufficient data ti determine demand rate.'].
	totalDemand := demandLog inject: 0 into:
		[:total :position | total + position quantity].
	totalDemand := totalDemand - demandLog first quantity.
	totalTime := demandLog last date 
					subtractDate: demandLog first date.
	^totalDemand / totalTime
	! !

Product categoriesFor: #add: are:((OrderedCollection new)
	add: 'Services';
	yourself)!
Product categoriesFor: #logOrderPosition: are:((OrderedCollection new)
	add: 'Accessing';
	yourself)!
Product categoriesFor: #initialize are:((OrderedCollection new)
	add: 'Initialize';
	yourself)!
Product categoriesFor: #availableQuantity are:((OrderedCollection new)
	add: 'Accessing';
	yourself)!
Product categoriesFor: #demandRate are:((OrderedCollection new)
	add: 'Accessing';
	yourself)!
Product categoriesFor: #printString are:((OrderedCollection new)
	add: 'Printing';
	yourself)!
Product categoriesFor: #remove: are:((OrderedCollection new)
	add: 'Services';
	yourself)!
Product categoriesFor: #unitVolume are:((OrderedCollection new)
	add: 'Accessing';
	yourself)!
Product categoriesFor: #name are:((OrderedCollection new)
	add: 'Accessing';
	yourself)!
Product categoriesFor: #unitVolume: are:((OrderedCollection new)
	add: 'Accessing';
	yourself)!
Product categoriesFor: #release are:((OrderedCollection new)
	add: 'Initialize';
	yourself)!
Product categoriesFor: #name: are:((OrderedCollection new)
	add: 'Accessing';
	yourself)!