
Object subclass: #TransportationUnit
    instanceVariableNames: 'customerOrder contents '
    classVariableNames: 'Volume '
    poolDictionaries: ''!

TransportationUnit comment: 'Diese Klasse entspricht der Klasse Transportbehaelter
aus der OBA-Analyse.

Instanzvariablen:
	customerOrder <CustomerOrder>
		Der ''Kundenauftrag'', dessen Produkte im Behaelter abgelegt sind.
	contens <IdentityDictionery>
		''Inhalt'' des Transportbehaelters.

Klassenvariablen:
	Volume <Number>
		Das ''Volumen'' eines einzelnen Transportbehaelters.'!

!TransportationUnit class publicMethods !

initialize
	"TransportUnit initialize."
	Volume := 100!

new
	"Instanzvariablen werden initialisiert."
	^super new initialize!

volume
	"Zugriff 'Volumen'."
	"Das Fassungsvermoegen wird zurueckgegeben."
	^Volume! !

TransportationUnit class categoriesFor: #new are:((OrderedCollection new)
	add: 'Create';
	yourself)!
TransportationUnit class categoriesFor: #initialize are:((OrderedCollection new)
	add: 'Create';
	yourself)!
TransportationUnit class categoriesFor: #volume are:((OrderedCollection new)
	add: 'Accessing';
	yourself)!

!TransportationUnit publicMethods !

customerOrder
	"Entspricht der Dienstleitung 'Kundenauftrag'."
	^customerOrder
!

customerOrder: aCustomerOrder
	"Der Kundenauftrag wird dem Transportbehaelter zugeordnet."
	customerOrder := aCustomerOrder!

freeSpace
	"Liefert das noch freie Packvolumen im Behaelter."
	| free |
	free := self class volume.
	contents keysAndValuesDo: [:product :amount |
									free := free - (product unitVolume * amount)].
	^free!

initialize
	"Anfangswerte werden gesetzt."
	customerOrder := nil.
	contents := IdentityDictionary new!

pack: aProduct amount: aNumber
	"Entspricht der Dienstleistung 'Kommissionieren'."
	"Es wird die Menge zurueckgegeben, die nicht
	in den Behaelter verpackt werden konnte."
	| free actualAmount currentAmount |
	(free := self freeSpace) < (aNumber * aProduct unitVolume)
		ifTrue: [actualAmount := free // aProduct unitVolume]
		ifFalse: [actualAmount := aNumber].
	currentAmount := contents at: aProduct ifAbsent:[0].
	contents at: aProduct put: currentAmount + actualAmount.
	^aNumber - actualAmount!

printString
	"Gibt den Inhalt eines Transportbehaelters aus."
	^'TransportUnit. Free: ' , self freeSpace printString!

release
	"Objektdaten werden geloescht."
	customerOrder := nil.
	contents := nil! !

TransportationUnit categoriesFor: #freeSpace are:((OrderedCollection new)
	add: 'Accessing';
	yourself)!
TransportationUnit categoriesFor: #release are:((OrderedCollection new)
	add: 'Initialize';
	yourself)!
TransportationUnit categoriesFor: #initialize are:((OrderedCollection new)
	add: 'Initialize';
	yourself)!
TransportationUnit categoriesFor: #printString are:((OrderedCollection new)
	add: 'Printing';
	yourself)!
TransportationUnit categoriesFor: #pack:amount: are:((OrderedCollection new)
	add: 'Services';
	yourself)!
TransportationUnit categoriesFor: #customerOrder are:((OrderedCollection new)
	add: 'Services';
	yourself)!
TransportationUnit categoriesFor: #customerOrder: are:((OrderedCollection new)
	add: 'Accessing';
	yourself)!

TransportationUnit initialize!